<form method="POST">
  <input type="HIDDEN" name="rm" value="post">
  <table>
    <tr>
      <td>タイトル:</td>
      <td><input type="text" name="subject" value="<%=h arg.subject%>" size="40"></td>
    </tr>
    <tr>
      <td>名前:</td>
      <td>
        <input type="text" name="from" value="<%=h arg.from%>" size="20">
        E-mail:<input type="text" name="mail" value="<%=h arg.mail%>" size="30">
      </td>
    </tr>
    <tr>
      <td>内容:</td>
      <td><textarea name="text" cols=60 rows=10 wrap="off"><%=h arg.text%></textarea></td>
    </tr>
  </table>
  <input type="submit" value="書き込む">
</form>
