<!DOCTYPE html PUBLIC "-//W3C//DTD HTML 4.01 Transitional//EN">
<html>
<head>
  <title><%=board_name%></title>
  <meta http-equiv="content-type" content="text/html; charset=EUC-JP">
  <meta name="author" content="MoonWolf">
  <meta name="description" content="simplebbs">
  <link rel="stylesheet" type="text/css" href="simplebbs.css">
</head>
<body>
<h1><%=board_name%></h1>
<p><%=form(arg)%></p>
<%
  arg.msglist.each {|msg|
    message(msg, io)
  }
  if arg.id_to > 1
%>
    <a href="<%=qs 'from'=>arg.id_to-1%>">Á°¤Î20·ï</a>
<%
  end
%>
<hr width="100%" size="2"><%=footer%><br>
</body>
</html>
