<p class="message">
<%=arg.msg_id%>
<span class="subject"><%=h arg.subject %></span>
by <% if arg.mail.empty? %>
     <%=h arg.name%>
   <% else %>
     <a href="mailto:<%=h arg.mail%>"><%=h arg.name%></a>
   <% end %>
on <%=h arg.msgdate %>
<br>
<span class="text"><%=msg2html(arg.message) %></span>
</p>
