# Web::Persistent::Serialize::Marshal
# Copyright(c) 2002 MoonWolf <moonwolf@moonwolf.com>
module Web
  class Persistent
    module Serialize
      module Marshal
        def self.dump(obj,port=nil)
          if port
            ::Marshal.dump(obj,port)
          else
            ::Marshal.dump(obj)
          end
        end 
        def self.restore(port)
          ::Marshal.restore(port)
        end
      end # Marshal
    end # Serialize
  end # Persistent
end # Web
