# Nora - Stray web libraries
# 
# Author::    MoonWolf  (mailto:moonwolf@moonwolf.com)
# Copyright:: Copyright (c) 2002 MoonWolf
# License::   Distributes under the same terms as Ruby

require 'web/common'
require 'web/cookie'
require 'web/request'
require 'web/response'
require 'web/persistent'
require 'web/interface'
