BODY {
	color : Black;
	background-color : #FFE4B5;
}
.message {
	background : White;
	border : 3px outset;
	padding : 0em 1em 0em 1em;
}
.subject {
	font-family : "ＭＳ Ｐゴシック";
	font-weight : bold;
	font : 16pt;
	color : #228B22;
}
.text {
	font : 11pt "ＭＳ ゴシック";
}